/*---------------------------------------------------------------------------

    Filename: graph.js
    Project:  Graphing Utilities
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         A control that graphically plots data sets and
                     allows for limited user interaction (e.g. zooming)
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/



var GraphPaste     = 1;
var GraphLegend    = 2;
var GraphZoom      = 4;
var GraphOverview  = 8;
var GraphLines     = 16;
var GraphPoints    = 32;
var GraphFill      = 64;
var GraphTimeSerie = 128;




class Graph extends Form
{
    // public events
    var areaSelected, // arguments: x1, y1, x2, y2
        plotClicked;  // arguments: x, y

    // private member variables
        // base variables
    var m_width, m_height, m_web,
        // option variables
        m_optionPaste, m_optionLegend, m_optionZoom, m_optionOverview,
        m_optionLines, m_optionPoints, m_optionFill, m_optionTimeserie,
        // event variables
        m_areaSelected, m_plotClicked;

    function Graph(options, x_pos, y_pos, width, height)
    {
        // set up the base member variables
        m_width = width ? width : 300;
        m_height = height ? height : 300;
        super("", x_pos, y_pos, m_width + 42, m_height + 64);

        m_optionPaste = options & GraphPaste;
        m_optionLegend = options & GraphLegend;
        m_optionOverview = options & GraphOverview;
        m_optionZoom = (options & GraphZoom) | m_optionOverview;
        m_optionLines = options & GraphLines;
        m_optionPoints = options & GraphPoints;
        m_optionFill = options & GraphFill;
        m_optionTimeserie = options & GraphTimeSerie;


        m_web = new WebBrowser(0, 0, m_width, m_height);
        this.add(m_web, 1, Layout.Expand);
        m_web.showHypertext(_getHtml());
        m_web.waitUntilReady();

        if(m_optionPaste)
        {
            // create the embedded "paste" image
            var element = m_web.getDOMDocument().getElementById("paste");
            var pasteEvent = new Event;
            pasteEvent.connect(this, onPaste);
            element.addEventListener("click", pasteEvent);
        }

        // set up custom events to intercept the events that can be handled 
        // externally
        var placeholder = m_web.getDOMDocument().getElementById("placeholder");

        m_areaSelected = new Event;
        m_areaSelected.connect(this, onAreaSelected);
        placeholder.addEventListener("areaselected", m_areaSelected);
        areaSelected = new Event;

        m_plotClicked = new Event;
        m_plotClicked.connect(this, onPlotClicked);
        placeholder.addEventListener("plotclicked", m_plotClicked);
        plotClicked = new Event;
    }
    
    // --- event handlers ---

    function onAreaSelected()
    {
        var element = m_web.getDOMDocument().getElementById("placeholder");

        var args = {
                       x1: element.getAttribute("areaselect_x1"),
                       y1: element.getAttribute("areaselect_y1"),
                       x2: element.getAttribute("areaselect_x2"),
                       y2: element.getAttribute("areaselect_y2")
                   };

        this.areaSelected.fire(this, args);
    }
    
    function onPlotClicked()
    {
        var element = m_web.getDOMDocument().getElementById("placeholder");

        var args = {
                       x: element.getAttribute("plotclick_x"),
                       y: element.getAttribute("plotclick_y")
                   };

        this.plotClicked.fire(this, args);
    }


    function onPaste()
    {
        _embedScript("var t = document.getElementById('pastebox');"
                    + " t.value=\"\"; t.focus();");
        if(!m_web.canPaste())
        {
            alert("Something went wrong (cannot paste)");
            return;
        }

        m_web.paste();

        _embedScript(_getJSCode_dataParser());
        
        _refreshGraph();
    }


    // --- public functions ---

    function setData(data)
    {
        var data_str = "";
        var numlines = data.length;
        
        for(var set = 1; set < data[0].length; set++)
        {
            data_str += "[";
            for(var line = 0; line < numlines; line++)
                data_str += "[" + (data[line][0]) + ",\""
                            + (data[line][set]) + "\"],";
            data_str += "],";
        }

        _embedScript("dataSet = [" + data_str + "];" + _getJSCode_mainOptions());
        _refreshGraph();
    }


    // --- private member functions

    function _refreshGraph()
    {
        if(m_optionZoom)
        {
            _embedScript(_getJSCode_zoom());
            if(m_optionOverview)
                _embedScript(_getJSCode_overview());
        }
         else
        {
            // --- begin raw javascript code ---
            _embedScript('
                if(pasteBox)
                    $(pasteBox).fadeTo("medium", dataSet ? 0.3 : 1.0);

                if(dataSet)
                    $.plot($("#placeholder"), dataSet, mainOptions);
            ');
            // --- end raw javascript code ---
        }
    }


    function _embedScript(code)
    {
        var document = m_web.getDOMDocument();
        var n = document.createElement("script");
        n.appendChild(document.createTextNode(code));

        var h = document.getElementsByTagName("head")[0];
        h.appendChild(n);
        h.removeChild(n);
    }
    


    function _getJSCode_mainOptions()
    {
        // --- begin raw javascript code ---
        return '
            var mainOptions = {
                legend: { show: ' + (m_optionLegend ? "true" : "false") + ' },
                lines: {
                        show: ' + (m_optionLines ? "true" : "false") + ',
                        fill: ' + (m_optionFill ? "true" : "false") + '
                    },
                colors: ["#F6B400", "#52B0F9"],
                points: { show: ' + (m_optionPoints ? "true" : "false") + ' },
                grid: { clickable: true },
                ' + (m_optionTimeserie ? "xaxis: {mode: \"time\"}," : "" ) + '
            };
        ';
        // --- end raw javascript code ---
    }
    
    
    
    function _getJSCode_dataParser()
    {
        // --- begin raw javascript code ---
        return '
            var t = document.getElementsByTagName("textarea")[0];
            var vals = t.value.split("\\n"), dataSet;
            t.value = "";
            t.setAttribute("numlines", vals.length);

            var rx = new RegExp("^(\\"?)([-+]?[0-9]*\\\\.[0-9]+|[0-9]+)\\\\1$");
            if(vals[0].match(rx))
            {
                dataSet = [[]];
                for(var i in vals)
                    dataSet[0][i] = [i, vals[i].replace(rx, "$2")];
            }
            else
            {
                dataSet = [];
                var numparts = vals[0].split("\\t").length + 1;

                for(var i = 0; i < numparts - 1; i++)
                    dataSet[i] = [];

                for(var line in vals)
                {
                    var parts = vals[line].substr(1, vals[line].length - 2)
                                          .split("\\"\\t\\"");
                    for(var i = 1; i < numparts; i++)
                        dataSet[i - 1][line] = [parts[0], parts[i]];
                }
            }
        ' + _getJSCode_mainOptions();
        // --- end raw javascript code ---
    }
    
    
    
    function _getJSCode_zoom()
    {
        // --- begin raw javascript code ---
        return '
        if(dataSet)
        {
            mainOptions.selection = { mode: "xy" };

            if(pasteBox)
                $(pasteBox).fadeTo("medium", 0.3);

            $.plot($("#placeholder"), dataSet, mainOptions);

            var internalSelection = false;

            $("#placeholder").bind("selected", function (event, area) {
                // clamp the zooming to prevent eternal zoom
                if (Math.abs(area.x2 - area.x1) < 0.00001)
                    area.x2 = area.x1 + 0.00001;
                if (Math.abs(area.y2 - area.y1) < 0.00001)
                    area.y2 = area.y1 + 0.00001;

                // do the zooming
                plot = $.plot($("#placeholder"), dataSet,
                              $.extend(true, {}, mainOptions, {
                                  xaxis: { min: area.x1, max: area.x2 },
                                  yaxis: { min: area.y1, max: area.y2 }
                              }));

                if (overview)
                {
                    if (internalSelection)
                        return; // prevent eternal loop
                    internalSelection = true;
                    overview.setSelection(area);
                    internalSelection = false;
                }
            });
        }
        ';
        // --- end raw javascript code ---
    }
    
    
    
    function _getJSCode_overview()
    {
        // --- begin raw javascript code ---
        return '
        if(dataSet[0].length)
        {
            var overview = $.plot($("#overview"), dataSet, {
                legend: { show: true, container: $("#overviewLegend") },
                lines: { show: true, lineWidth: 1 },
                shadowSize: 0,
                xaxis: { ticks: 4 },
                yaxis: { ticks: 2 },
                grid: { color: "#999" },
                selection: { mode: "xy" }
            });

            $("#overview").bind("selected", function (event, area) {
                if (internalSelection)
                    return;
                internalSelection = true;
                plot.setSelection(area);
                internalSelection = false;
            });

        }
        ';
        // --- end raw javascript code ---
    }



    function _getHtml()
    {
        var width = m_optionOverview ? m_width - 200 : m_width,
            height = m_height;

        var jscode1 = Extension.getTextResource("flot/jquery.js");
        if(jscode1 == null)
            throw "Could not load jquery.js source code";

        var jscode2 = Extension.getTextResource("flot/jquery.flot.pie.js");
        if(jscode2 == null)
            throw "Could not load jquery.flot.pie.js source code";




        var str = "<html><head><script type=\"text/javascript\">\n"
            + jscode1 + "\n" + jscode2 + "\n"

        
        // --- begin raw HTML string ---
        + '
var dataSet = null;

</script>
</head>
<body style="padding: 0; margin: 0; overflow: hidden;">

<textarea id="pastebox"
style="position: absolute; top: -99999px; width: 0px; height: 0px;">
</textarea>

<script type="text/javascript">
var overview = null;
var pasteBox = null;

var placeholder = document.createElement("div");
placeholder.id = "placeholder";
placeholder.style.cssFloat = "left";
placeholder.style.height = "' + height + 'px";
placeholder.style.width = "' + width + 'px";
document.body.appendChild(placeholder);
$(placeholder).bind("selected", function (event, area) {

    placeholder.setAttribute("areaselect_x1", area.x1);
    placeholder.setAttribute("areaselect_y1", area.y1);
    placeholder.setAttribute("areaselect_x2", area.x2);
    placeholder.setAttribute("areaselect_y2", area.y2);

    var evObj = document.createEvent("MouseEvents");
    evObj.initEvent("areaselected", true, true);
    this.dispatchEvent(evObj);
});
$(placeholder).bind("plotclick", function (event, position) {

    placeholder.setAttribute("plotclick_x", position.x);
    placeholder.setAttribute("plotclick_y", position.y);

    var evObj = document.createEvent("MouseEvents");
    evObj.initEvent("plotclicked", true, true);
    this.dispatchEvent(evObj);
});


        ';
        // --- end raw HTML string ---    




        if(m_optionZoom) // TODO: fix the bug in this part
        // --- begin raw HTML string ---
            str += '
var zoomBtn = document.createElement("div");
zoomBtn.id = "zoom";
zoomBtn.style.height = "16px";
zoomBtn.style.width = "16px";
zoomBtn.style.top = "' + (height - 16) + 'px";
zoomBtn.style.left = "' + (width - 34) + 'px";
zoomBtn.style.cursor = "pointer";
zoomBtn.style.position = "absolute";
zoomBtn.style.zIndex = "500";
/*zoomBtn.style.background = "url(data:image/png;base64,"
    + "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQBAMAAADt3eJSAAAALVBMVEUAAAAAAABwcDhwcGB"
    + "4eGgAAICAgAC/vwCAgECAgHDLywyAgIDAwMDPz9////+gxUtPAAAAAXRSTlMAQObYZgAAAG"
    + "ZJREFUeNpjYEAARkEBCCW4ShDEFNsmLli4bSIDg/TGM2fOCCcCRRwFBQWlNzIwSHR0ZmdnJ"
    + "4IYHaGhoYFgRty7d6EwRmwAmBEaFxoKZGgARd49ZZDs6ATyQ5+CrICIAEEoCMDdAABnVibO"
    + "b3ulgwAAAABJRU5ErkJggg==)";*/
document.body.appendChild(zoomBtn);
$(zoomBtn).hide();
$(zoomBtn).bind("mouseenter", function(e) { $(this).fadeTo("medium", 1.0); });
$(zoomBtn).bind("mouseleave", function(e) {
        $(this).fadeTo("medium", dataSet == null ? 1.0 : 0.33); });
            ';
        // --- end raw HTML string ---




        if(m_optionPaste)
        // --- begin raw HTML string ---
            str += '
var pasteBox = document.createElement("div");
pasteBox.id = "paste";
pasteBox.style.height = "16px";
pasteBox.style.width = "16px";
pasteBox.style.top = "' + (height - 16) + 'px";
pasteBox.style.left = "' + (width - 16) + 'px";
pasteBox.style.cursor = "pointer";
pasteBox.style.position = "absolute";
pasteBox.style.zIndex = "500";
pasteBox.style.background = "url(data:image/png;base64,"
    + "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQBAMAAADt3eJSAAAALVBMVEUAAAAAAABwcDhwcGB"
    + "4eGgAAICAgAC/vwCAgECAgHDLywyAgIDAwMDPz9////+gxUtPAAAAAXRSTlMAQObYZgAAAG"
    + "ZJREFUeNpjYEAARkEBCCW4ShDEFNsmLli4bSIDg/TGM2fOCCcCRRwFBQWlNzIwSHR0ZmdnJ"
    + "4IYHaGhoYFgRty7d6EwRmwAmBEaFxoKZGgARd49ZZDs6ATyQ5+CrICIAEEoCMDdAABnVibO"
    + "b3ulgwAAAABJRU5ErkJggg==)";
document.body.appendChild(pasteBox);
$(pasteBox).bind("mouseenter", function(e) { $(this).fadeTo("medium", 1.0); });
$(pasteBox).bind("mouseleave", function(e) {
        $(this).fadeTo("medium", dataSet ? 0.3 : 1.0); });
            ';
        // --- end raw HTML string ---


        str += "</script>";


        if(m_optionOverview)
        // --- begin raw HTML string ---
            str += '
<div id="miniature" style="float:left;margin-left:20px;margin-top:50px">
    <div id="overview" style="width: 166px; height: '
        + Math.round(166 * (height / width)) + 'px;"></div>
    <p id="overviewLegend" style="margin-left:10px"></p>
</div>
            ';
        // --- end raw HTML string ---



        str += "\n</body>\n</html>";

        return str;
    }
}





class LineGraph extends Graph
{
    function LineGraph(options, x_pos, y_pos, width, height)
    {
        options |= GraphLines; // set GraphLines = true
        super(options, x_pos, y_pos, width, height);
    }
}



class PointGraph extends Graph
{
    function PointGraph(options, x_pos, y_pos, width, height)
    {
        options |= GraphLines; // set GraphLines = false
        options -= GraphLines;
        options |= GraphPoints; // set GraphPoints = true
        super(options, x_pos, y_pos, width, height);
    }
}



class BarGraph extends Graph
{
    function BarGraph(options, x_pos, y_pos, width, height)
    {
        options |= GraphFill; // set GraphFill = true
        options |= GraphPoints; // set GraphPoints = false
        options -= GraphPoints;
        options |= GraphLines; // set GraphLines = true
        super(options, x_pos, y_pos, width, height);
    }

    function _getJSCode_dataParser()
    {
        // --- begin raw javascript code ---
        return '
            var t = document.getElementsByTagName("textarea")[0];
            var vals = t.value.split("\\n"), dataSet;
            t.value = "";
            t.setAttribute("numlines", vals.length);

            var rx = new RegExp("^(\\"?)([-+]?[0-9]*\\\\.[0-9]+|[0-9]+)\\\\1$");
            if(vals[0].match(rx))
            {
                dataSet = [[]];
                for(var i in vals)
                {
                    var val = vals[i].replace(rx, "$2");
                    dataSet[0].push([2 * i, 0]);
                    dataSet[0].push([2 * i, val]);
                    dataSet[0].push([2 * i + 1.5, val]);
                    dataSet[0].push([2 * i + 1.5, 0]);
                    dataSet[0].push(null);
                }
            }
            else
            {
                dataSet = [];
                var numparts = vals[0].split("\\t").length + 1;

                for(var i = 0; i < numparts; i++)
                    dataSet[i] = [];

                var x = 0;
                for(var line in vals)
                {
                    var parts = vals[line].substr(1, vals[line].length - 2)
                                          .split("\\"\\t\\"");
                    for(var i = 0; i < numparts; i++)
                    {
                        dataSet[i].push([x, 0]);
                        dataSet[i].push([x, parts[i]]);
                        dataSet[i].push([x + 1, parts[i]]);
                        dataSet[i].push([x + 1, 0]);
                        dataSet[i].push(null);
                        x++;
                    }
                }
            }

        ' + _getJSCode_mainOptions();
        // --- end raw javascript code ---
    }
}



class PieGraph extends Graph
{
    function PieGraph(options, x_pos, y_pos, width, height)
    {
        options |= GraphOverview; // set GraphOverview = false
        options -= GraphOverview;
        options |= GraphZoom; // set GraphZoom = false
        options -= GraphZoom;
        super(options, x_pos, y_pos, width, height);
    }

    function setData(data)
    {
        var data_str = "";
        var numlines = 0;

        for(var i in data)
        {
            data_str += "{ label: \"" + (+i == i ? "Series " + i : i)
                        + "\", data: " + data[i] +"},";
            numlines++;
        }

        _embedScript("dataSet = [" + data_str + "];" + _getJSCode_mainOptions());
        _refreshGraph();
    }

    function _getJSCode_mainOptions()
    {
        // --- begin raw javascript code ---
        return '
            var mainOptions = {
                legend: { show: ' + (m_optionLegend ? "true" : "false") + ' },
                pie: {
                    show: true, 
                    pieStrokeLineWidth: 1, 
                    pieStrokeColor: "#FFF", 
                    showLabel: true
                }
            };
        ';
        // --- end raw javascript code ---
    }

    function _getJSCode_dataParser()
    {
        // --- begin raw javascript code ---
        return '
            var t = document.getElementsByTagName("textarea")[0];
            var vals = t.value.split("\\n"), dataSet;
            t.value = "";
            t.setAttribute("numlines", vals.length);
            
            dataSet = [];

            var rx = new RegExp("^(\\"?)([0-9]*\\\\.[0-9]+|[0-9]+)\\\\1$");
            if(vals[0].match(rx))
            {
                for(var i in vals)
                    dataSet[i] = {
                                    label: "Serie" + (i+1),
                                    data: vals[i].replace(rx, "$2")
                                 };
            }

        ' + _getJSCode_mainOptions();
        // --- end raw javascript code ---
    }
}
