/*---------------------------------------------------------------------------

    Filename: main.js
    Project:  Benford Analysis
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         A utility for graphing a set of data against a Benford curve
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/



include "graph.js";
include "utils/tools_menu_inserter.js";


var g_gcb = [];

// main entry point to the program; launch the program, either immediately if
// run as as script or else install to the tools menu if run as an extension
if (Extension.isContextPackage())
{
    // program is running as an extension, install to the "Tools" menu
    var menu_item = insertToolsMenuItem("Benford Analysis...");

    // when triggered, display the main form
    menu_item.click.connect(
        function()
        {
            // display the window
            g_gcb.push(new BenfordAnalysis(640, 480));
        });
}
 else
{
    // program is running as a script, display the form immediately
    new BenfordAnalysis(640, 480);
}

// start the main application loop
Application.run();




// main window that contains the graph and the other controls selecting a source
class BenfordAnalysis extends Form
{
    function BenfordAnalysis(width, height)
    {
        super("Benford Analysis", 0, 0, width, height);
        this.setMinSize(width, height);
        this.setMaxSize(width, height);
        m_db = new DbConnection;

        // create the components that belong to this form
        _setupLayout();

        // create the initial graph
        _regraph();
    }

    function _setupLayout()
    {

        // layout information for the top controls

        m_sourcetableTextbox = new TextBox();
        m_sourcetableTextbox.enterPressed.connect(this, sourceTableEnterPressed);

        var browseTablesButton = new Button("Browse...");
        browseTablesButton.click.connect(this, sourceTableBrowseClicked);

        m_sourcecolumnChoicebox = new ChoiceBox(0, 0, 100);

        m_digitsSpinbox = new SpinBox(0, 0, 45);
        m_digitsSpinbox.setRange(1, 3);

        m_graphBtn = new Button("Graph");
        m_graphBtn.click.connect(this, _regraph);

        var topLayout = new BoxLayout(Layout.Horizontal);
        topLayout.add(new Label("Source Table: "), 0, Layout.Center);
        topLayout.add(m_sourcetableTextbox, 1, Layout.Center);
        topLayout.addSpacer(8);
        topLayout.add(browseTablesButton, 0, Layout.Center);
        topLayout.addSpacer(16);
        topLayout.add(new Label("Column: "), 0, Layout.Center);
        topLayout.add(m_sourcecolumnChoicebox, 0, Layout.Center);
        topLayout.addSpacer(16);
        topLayout.add(new Label("Digits: "), 0, Layout.Center);
        topLayout.add(m_digitsSpinbox, 0, Layout.Center);
        topLayout.addSpacer(16);
        topLayout.add(m_graphBtn, 0, Layout.Expand);


        // layout information for the graph

        m_graphLayout = new BoxLayout(Layout.Vertical);

        var bottomForm = new Form();
        bottomForm.setBackgroundColor(new Color(255, 255, 255));
        bottomForm.setLayout(m_graphLayout);


        // construct the primary layout

        var mainLayout = new BoxLayout(Layout.Vertical);
        mainLayout.addSpacer(8);
        mainLayout.add(topLayout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        mainLayout.addSpacer(8);
        mainLayout.add(new Line(), 0, Layout.Expand);
        mainLayout.add(bottomForm, 1, Layout.Expand);


        // display this form

        this.setLayout(mainLayout);
        this.center();
        this.show();
    }


    // --- event handlers ---

    function sourceTableBrowseClicked()
    {
        // display a dialog to allow the user to select a table
        var dialog = new ProjectFileDialog(ProjectFileDialog.OpenFile);
        dialog.setCaption("Select a source table");

        // if a table was selected, then put that path in the table textbox and
        // update the column choicebox
        if (dialog.showDialog() == true)
        {
            m_sourcetableTextbox.setText(dialog.getPath());
            sourceTableEnterPressed();
        }
    }

    function sourceTableEnterPressed()
    {
        // clear out the contents of the column choicebox
        m_sourcecolumnChoicebox.clear();
        m_sourcecolumnChoicebox.add("");
        m_sourcecolumnChoicebox.select(0);

        // get the fields that belong to the table;
        // if the table does not exist, then return
        var fields = m_db.describeTable(m_sourcetableTextbox.getText());
        if (!fields)
            return false;

        // freeze
        this.disablePaint();

        // add all of the columns from the table into the column choicebox
        for (var i in fields)
            m_sourcecolumnChoicebox.add(fields[i].name);

        // thaw
        this.enablePaint();
    }

    function _regraph()
    {
        // temporarily hide the graph while the data is generated and the
        // points are plotted; instead display the text "Graphing"
        this.disablePaint();
        m_graphingLabel = new Label("Processing (99%)");
        m_graphingLabel.setFont(new Font(Font.Default, 20));
        m_graphingLabel.setForegroundColor(new Color(175, 175, 175));
        m_graphLayout.clear();
        m_graphLayout.layout();
        m_graphLayout.addStretchSpacer();
        m_graphLayout.add(m_graphingLabel, 0, Layout.Center);
        m_graphLayout.addStretchSpacer();
        m_graphLayout.layout();
        this.enablePaint();


        // create the graph object, sizing it to the current window's dimensions
        var dimensions = this.getClientSize();
        var graph = new LineGraph(undefined, 0, 0, dimensions.width - 24,
                                        dimensions.height - 65);


        // generate and graph the data
        var data = _generateData();
        graph.setData(data);


        // hide the "Graphing" text and display the graph
        this.disablePaint();
        m_graphLayout.clear();
        m_graphLayout.layout();
        m_graphLayout.add(graph, 1, Layout.Expand | Layout.All, 12);
        m_graphLayout.layout();
        this.enablePaint();
    }

    function _generateData()
    {
        var numdigits = m_digitsSpinbox.getValue();

        // the range of the x-axis will be {lowcutoff <= x < base}
        var base = Math.pow(10, numdigits);
        var lowcutoff = base / 10;

        // array to store the generated plot points
        var data = undefined;


        // the user selected a column so generate 2 lines, one as the base
        // (Benford) line and the second based on the input data from the
        // selected table column
        if (m_sourcecolumnChoicebox.getSelection() > 0)
        {
            // fetch the total number of records in the table
            var currentProgress = 0, totalRecords = 0;
            var result = m_db.execute("SELECT COUNT(*) FROM "
                                    + m_sourcetableTextbox.getText());
            if (result && result.next())
                totalRecords = result.getValue(0);


            // completely fill out the data array with zeros
            var tabledata = new Array, total = 0, numRecords = totalRecords;
            for (var d = lowcutoff; d < base; d++)
                tabledata[d] = 0;

            // fetch the raw data from the table
            result = m_db.execute("SELECT "
                            + m_sourcecolumnChoicebox.getString()
                            + " FROM " + m_sourcetableTextbox.getText());
            if (!result)
            {
                alert("Sorry, the data could not be loaded.  Please make sure "
                    + "your table name and column names do not contain spaces.");
            }

            // process every record in the table
            while(result && result.next())
            {
                // strip out everything but the first n digits
                var digits = +("" + Math.abs(Math.floor(+result.getValue(0))))
                              .substr(0, numdigits);

                // add to the count using only the first n digits
                if (digits >= lowcutoff && digits <= base)
                {
                    tabledata[digits]++;
                    total++;
                }
                // if the number is zero, do not include it in calculations
                else if (digits < lowcutoff)
                    numRecords--;

                // update the graphing label every 1000 records
                currentProgress++;
                if (currentProgress % 1000 == 1)
                    m_graphingLabel.setLabel("Processing ("
                                        + Math.round(100 * currentProgress/totalRecords)
                                        + "%)");
            }

            // if all of the data was added correctly, generate the plot data
            if (total)
            {
                // add the table data as percentages to to the base data
                data = [[0, null, null]];
                for (var d = lowcutoff; d < base; d++)
                    // generate the plotpoints as [x, y1, y2] pairs
                    data.push([ d, // digit
                                numdigits * Math.log(1 + (1/d)) / Math.log(base), // base line
                                tabledata[d] / total // table data
                               ]);
            }
        }

        // if no column was selected, or a line could not be generated using
        // the selected table column, only create the base (Benford) line
        if (!data)
        {
            data = [[0, null]];

            // generate the data as [x, y] pairs
            for (var d = lowcutoff; d < base; d++)
                data.push([d, numdigits * Math.log((d + 1) / d) / Math.log(base)]);
        }


        // reset the graphing label because the progress can no longer be
        // determined
        this.disablePaint();
        m_graphingLabel.setLabel("  Graphing...");
        m_graphLayout.layout();
        this.enablePaint();


        return data;
    }

    // instance member variables
    var m_db;
    // layout variables
    var m_sourcetableTextbox, m_sourcecolumnChoicebox, m_graphBtn,
        m_digitsSpinbox, m_graphLayout, m_graphingLabel;
}
